#!/usr/bin/python

#####################################################################################
# pico_status_hv4.0.py
# Version 1.2 Relase Date 04.09.2022
# Script to show you some statistics pulled from your UPS PIco HV4.0
# -*- coding: utf-8 -*-
# re-Written by Ioannis A. Mourtsiadis info@pimodules.com
#####################################################################################

# Set your desired temperature symbol
# C = Celsius
# F = Fahrenheit
degrees = "C"

import sys
import time
import datetime
import os
import re     
import getopt
import smbus
#import psutil
import time
import datetime
import threading
import RPi.GPIO as GPIO

i2c = smbus.SMBus(1)

############### FUNCTIONS #################

def fw_version():
   time.sleep(0.1)
   data = i2c.read_word_data(0x69, 0x38)
   data = format(data,"04x")
   return data
   
def pcb_version():
   time.sleep(0.1)
   data = i2c.read_byte_data(0x69, 0x35)
   data = format(data,"02x")
   return data   
   
def PIco_model():
   time.sleep(0.1)
   data = i2c.read_byte_data(0x69, 0x36)
   data = format(data,"02x")
   return data
   
def pcb_battery():
   time.sleep(0.1)
   data = i2c.read_byte_data(0x69, 0x37)
   if (data == 0x4C):
      return "Battery LiPO 'L'	0x4C"
   elif (data == 0x49):
      return "Battery Li-Ion 'I'	0x49"
   elif (data == 0x46):
      return "Battery LiFePO4 'F'	0x46"
   elif (data == 0x48):
      return "Battery NiMH 'H'	0x48"
   elif (data == 0x41):
      return "Battery SAL 'A'	0x41"
   elif (data == 0x43):
      return "On Board Super Cap (100F) 'C' 0x43"
   elif (data == 0x44):
      return "External Super Cap HAT 'D' (300F-500F-800F) 0x44"
   elif (data == 0x6C):
      return "Battery LiPO+SC 'l' 0x6C"
   elif (data == 0x69):
      return "Battery Li-Ion+SC	'i'	0x69"
   elif (data == 0x66):
      return "Battery LiFePO4+SC 'f' 0x66"
   elif (data == 0x68):
      return "Battery NiMH+SC 'h' 0x68"
   elif (data == 0x61):
      return "Battery SAL+SC 'a' 0x61"
   else:
      return "ERROR" 
  
def default_battery():
   time.sleep(0.1)
   data = i2c.read_byte_data(0x6b, 0x07)
   if (data == 0x4C):
      return "Battery LiPO 'L'	0x4C"
   elif (data == 0x49):
      return "Battery Li-Ion 'I'	0x49"
   elif (data == 0x46):
      return "Battery LiFePO4 'F'	0x46"
   elif (data == 0x48):
      return "Battery NiMH 'H'	0x48"
   elif (data == 0x41):
      return "Battery SAL 'A'	0x41"
   elif (data == 0x43):
      return "On Board Super Cap (100F) 'C' 0x43"
   elif (data == 0x44):
      return "External Super Cap HAT 'D' (300F-500F-800F) 0x44"
   elif (data == 0x6C):
      return "Battery LiPO+SC 'l' 0x6C"
   elif (data == 0x69):
      return "Battery Li-Ion+SC	'i'	0x69"
   elif (data == 0x66):
      return "Battery LiFePO4+SC 'f' 0x66"
   elif (data == 0x68):
      return "Battery NiMH+SC 'h' 0x68"
   elif (data == 0x61):
      return "Battery SAL+SC 'a' 0x61"
   else:
      return "ERROR"
      
def getCPUtemperature():
    time.sleep(0.1)
    res = os.popen('vcgencmd measure_temp').readline()
    cpu_data=res.replace("temp=","").replace("'C\n","")
    cpu_int=int(float(cpu_data))
    return format(cpu_int,"02d")
    
def getFANThreshold():
   data = i2c.read_byte_data(0x6b, 0x13)
   data = format(data,"02x")
   return data 
    
def fan_state():
   time.sleep(0.1)
   data = i2c.read_byte_data(0x69, 0x21)
   #data = format(data,"02d")
   if (data & 0x01 == 0x01):
      return "ON"
   elif (data == 0):
      return "OFF"
   else:
      return data
      
def pwr_mode():
   data = i2c.read_byte_data(0x69, 0x00)
   data = data & ~(1 << 7)
   if (data == 1):
      return "Cable POWERED"
   elif (data == 0):
      return "BAT/SCAP POWERED"
   else:
      return "ERROR"
      
def backing_mode():
   data = i2c.read_byte_data(0x69, 0x01)
   #data = data & ~(1 << 7)
   if (data == 0x01):
      return "Battery Backup"
   elif (data == 0x02):
      return "Super Capacitor Backup"
   elif (data == 0x03):
      return "Mixed Bat/SCap Backup"       
   else:
      return "ERROR"
   
def sca_read():
   time.sleep(0.1)
   data = i2c.read_word_data(0x69, 0x06)
   data = format(data,"02x")
   return (float(data) / 100) 

def bat_read():
   time.sleep(0.1)
   data = i2c.read_word_data(0x69, 0x08)
   data = format(data,"02x")
   return (float(data) / 100) 

def rpi_read():
   time.sleep(0.1)
   data = i2c.read_word_data(0x69, 0x0a)
   data = format(data,"02x")
   return (float(data) / 100) 

def epr_read():
   time.sleep(0.1)
   data = i2c.read_word_data(0x69, 0x0c)
   data = format(data,"02x")
   return (float(data) / 100)
   
def rpi_hold_on():
   time.sleep(0.1)
   data = i2c.read_word_data(0x6b, 0x03)
   data = format(data,"02x")
   return data   
 
def bat_runtime():
   time.sleep(0.1)
   data = i2c.read_byte_data(0x6b, 0x01)
   if (data == 0xff):
      return "Unlimited - Timer Disabled"      
   else:
      return data

def relay_state():
   time.sleep(0.1)
   data = i2c.read_byte_data(0x6b, 0x0b)
   if (data & 0x01 == 0x01):
      return "ON"      
   else:
      return "OFF"
      
def pico_rpi_route():
   data = i2c.read_byte_data(0x6b, 0x04)
   if (data == 0x00):
      return "none"
   if (data == 0x01):
      return "Raspberry Pi RS232 Channel 0"
   if (data == 0x02):
      return "Raspberry Pi RS232 Channel 2"
   if (data == 0x03):
      return "Raspberry Pi RS232 Channel 4"   
   if (data == 0x04):
      return "Raspberry Pi RS232 Channel 5"    
    
   return "ERROR"      
      
def RS232_rate():
   time.sleep(0.1)
   data = i2c.read_byte_data(0x6b, 0x02)     
   map="" 
   if(data & 0x0f == 0x00):
      map="RPI_RS232_OFF     "
   if(data == 0x01):     
      map="RPI_RS232_4800    "
   if(data == 0x02):     
      map="RPI_RS232_9600    "
   if(data == 0x03):
      map="RPI_RS232_19200   "
   if(data == 0x04):
      map="RPI_RS232_38400   "
   if(data == 0x05):
      map="RPI_RS232_57600   "
   if(data == 0x0f):
      map="RPI_RS232_115200  "

 
   if(data & 0xf0 == 0x00):
      map=map+"PICO_RS232_OFF"
   if(data == 0x10):     
      map=map+"PICO_RS232_4800"
   if(data == 0x20):     
      map=map+"PICO_RS232_9600"
   if(data == 0x30):
      map=map+"PICO_RS232_19200"
   if(data == 0x40):
      map=map+"PICO_RS232_38400"
   if(data == 0x50):
      map=map+"PICO_RS232_57600"
   if(data == 0xf0):
      map=map+"PICO_RS232_115200"

   return map
      
def sta_runtime():
   time.sleep(0.1)
   data = i2c.read_byte_data(0x6b, 0x05)
   if (data == 0xff):
      return "Timer Disabled"      
   else:
      return data

def fssd_duration():
   time.sleep(0.1)
   data = i2c.read_byte_data(0x6b, 0x1a)
   return data  
   
def enable5V():
   time.sleep(0.1)
   data = i2c.read_byte_data(0x6b, 0x06)
   if (data & 0x01 == 0x01):
      return "ON"      
   else:
      return "OFF"

def on_the_go():
   time.sleep(0.1)
   data = i2c.read_byte_data(0x6b, 0x16)
   if (data == 0xaa):
      return "Activated"      
   else:
      return "de-Activated"  

def rpi_OFF_time_reg():
   time.sleep(0.1)
   data = i2c.read_byte_data(0x6b, 0x1b)
   return data  
      
def pico_is_running():
   time.sleep(0.1)
   data0 = i2c.read_word_data(0x69, 0x04)
   time.sleep(0.5)
   data1 = i2c.read_word_data(0x69, 0x04)
   if(data0 == data1):
      return "Faulty - Not Running"
   else:
      return "Healthy - Running Properly"
      
      
def led_green():
   time.sleep(0.1)   
   data = i2c.read_byte_data(0x6b, 0x09)  
   if (data & 0x01 == 0x01):
      return "ON"      
   else:
      return "OFF"   

def led_blue():
   time.sleep(0.1)   
   data = i2c.read_byte_data(0x6b, 0x0a)  
   if (data & 0x01 == 0x01):
      return "ON"      
   else:
      return "OFF"   

def map_led_blue():
   time.sleep(0.1) 
   data = i2c.read_byte_data(0x6b, 0x17)
   if (data == 0x00):
      return "none"  
      
   map="" 
   if (data & 0x01 == 0x01):
      map=map+"SYS "
      
   if (data & 0x02 == 0x02):
      map=map+"BAT "  
      
   if (data & 0x04 == 0x04):
      map=map+"CHG "
      
   if (data & 0x08 == 0x08):
      map=map+"TMP "
      
   if (data & 0x10 == 0x10):
      map=map+"SCA "
      
   if (data & 0x20 == 0x20):
      map=map+"REL "
      
   if (data & 0x40 == 0x40):
      map=map+"TMR "
      
   if (data & 0x80 == 0x80):
      map=map+"CBL "
   
   return map  
                 

def map_led_green():
   time.sleep(0.1) 
   data = i2c.read_byte_data(0x6b, 0x18)
   if (data == 0x00):
      return "none"  
      
   map="" 
   if (data & 0x01 == 0x01):
      map=map+"SYS "
      
   if (data & 0x02 == 0x02):
      map=map+"BAT "  
      
   if (data & 0x04 == 0x04):
      map=map+"CHG "
      
   if (data & 0x08 == 0x08):
      map=map+"TMP "
      
   if (data & 0x10 == 0x10):
      map=map+"SCA "
      
   if (data & 0x20 == 0x20):
      map=map+"REL "
      
   if (data & 0x40 == 0x40):
      map=map+"TMR "
      
   if (data & 0x80 == 0x80):
      map=map+"CBL "
   
   return map  
   
def BAT_chg_stat():
   time.sleep(0.1)
   data = i2c.read_byte_data(0x69, 0x1e)
   data = format(data,"02d")
   return (float(data) / 100)
   
def SCA_pwm_stat():
   time.sleep(0.1)
   data = i2c.read_byte_data(0x69, 0x20)
   data = format(data,"02d")
   data = float(data)*1.27
   return (data / 100)
   
def chg_ctrl():
   time.sleep(0.1)   
   data = i2c.read_byte_data(0x6b, 0x19)  
   if (data == 0x01):
      return "ON"      

   if (data == 0x00):
      return "OFF"  
      
   if (data == 0xCC):
      return "Charging during LPR"  

   return "ERROR"        

def ad_corectionf():
   time.sleep(0.1)
   data = i2c.read_byte_data(0x6b, 0x1c)
   data = format(data,"02d")
   return data 

def ad0_read():
   time.sleep(0.1)
   data = i2c.read_word_data(0x69, 0x14)
   data = format(data,"02x")
   return (float(data) / 100)
   
def ad1_read():
   time.sleep(0.1)
   data = i2c.read_word_data(0x69, 0x16)
   data = format(data,"02x")
   return (float(data) / 100)
   
def ecilevel():
   time.sleep(0.1)
   data = i2c.read_word_data(0x69, 0x12)
   data = format(data,"02x")
   return (float(data) / 100)

   

print (" ")
print ("**************************************")
print ("        UPS PIco HV4.0 Status         ")
print (" Version 1.2   Relase Date 04.09.2022 ")
print ("Compatible with Firmware 0x0140 and up")
print ("**************************************")
print (" ")
print (" ","UPS PIco Firmware..........................:",fw_version())
print (" ","UPS PIco Bootloader........................:",pcb_version())
print (" ","UPS PIco PCB Version.......................:",PIco_model())
print (" ","UPS PIco PCB Defaut Battery................:",pcb_battery())
print (" ","UPS PIco Set Battery.......................:",default_battery())
print (" ")
print (" ","Raspberry Pi Core Temprature...............:",int(getCPUtemperature(),10), "C")
print (" ","Embedded Fan Temp. Threshold...............:",getFANThreshold(), "C")
print (" ","Embedded Fan Speed.........................:",fan_state(),"% of Total RPM")
print (" ")
print (" ","Powering Mode..............................:",pwr_mode())
print (" ","Power Backup Set Mode......................:",backing_mode())
print (" ")
print (" ","Battery Level..............................:",bat_read(), "V")
print (" ","Super Capacitor Level......................:",sca_read(), "V")
print (" ","EPR/PPoE Powering Level....................:",epr_read(), "V")
print (" ","RPi(USB) Powering Level....................:",rpi_read(), "V")
print (" ")
print (" ","Running Time on Battery....................:",bat_runtime(), "min")
print (" ","Battery Charger Status.....................:",chg_ctrl())
print (" ","Max Allowed Battery charging curent........:",BAT_chg_stat(), " A")
print (" ","Max Allowed On Board SCAP charging curent..:",SCA_pwm_stat(), " A")
#print (" ","Current Source of Power Backup..:",backup_source())
print (" ")
print (" ","Startup on Hold time.'.....................:",rpi_hold_on(),"Seconds")
print (" ","FSSD Durtion...............................:",fssd_duration(),"Seconds")
print (" ","Firmware loop..............................:",pico_is_running())
print (" ","Still Alive Timer..........................:",sta_runtime())
print (" ","Raspberry Pi OFF waiting time..............:",rpi_OFF_time_reg(),"min")
print (" ","Magic Switch Funtionality..................:",on_the_go())
print (" ")
print (" ","UPS PIco HV4.0 RS232(s) Setup..............:",RS232_rate())
print (" ","-12/+12 V RS232 Driver Connection..........:",pico_rpi_route())
print (" ","RS232 PIco routing.........................:","not implemented yet")
print (" ","RS232(s) PIco sleep/wakeup data setup......:","not implemented yet")
print (" ","RS232(s) PIco sleep/wakeup data received...:","not implemented yet")
print (" ")
print (" ","Internal Relay State.......................:",relay_state())
print (" ","User LED Green Activity....................:",led_green())
print (" ","User LED Blue Activity.....................:",led_blue())
print (" ","Green LED mapping setup....................:",map_led_green())
print (" ","Blue LED mapping setup.....................:",map_led_blue()) 

   
#   unsigned int8    LED_OFF;                 /// (0x15 b)    LEDs ON =0, LEDS OFF =1;
  
print (" ")
print (" ","A/D Converters Correction Factor...........:",ad_corectionf()) 
print (" ","A/D Converter 0 ...........................:",ad0_read())
print (" ","A/D Converter 1 ...........................:",ad1_read())
print (" ","Opto Coupler/Digital Input Level...........:",ecilevel())
print (" ")
print ("**************************************")
print ("        Powered by PiModules          ")
print ("**************************************")
print (" ")